pub type EncapsKey = EncapsKey<EK_LEN>;Expand description
Correctly sized encapsulation key specific to the target security parameter set.
Aliased Type
#[repr(align(8))]pub struct EncapsKey(/* private fields */);Trait Implementations
Sourceimpl Encaps for EncapsKey
impl Encaps for EncapsKey
Sourcetype CipherText = CipherText<CT_LEN>
type CipherText = CipherText<CT_LEN>
The ciphertext type transmitted from the encapsulating party to the decapsulating party
The shared secret key type generated during encapsulation
Sourcefn try_encaps_with_rng(
&self,
rng: &mut impl CryptoRngCore,
) -> Result<(Self::SharedSecretKey, Self::CipherText), &'static str>
fn try_encaps_with_rng( &self, rng: &mut impl CryptoRngCore, ) -> Result<(Self::SharedSecretKey, Self::CipherText), &'static str>
Generates a shared secret and ciphertext from an encapsulation key specific to this security parameter set.
This function utilizes a provided random number generator and is intended to operate in constant time. Read more
This function utilizes a provided random number generator and is intended to operate in constant time. Read more
Sourcefn try_encaps(
&self,
) -> Result<(Self::SharedSecretKey, Self::CipherText), &'static str>
fn try_encaps( &self, ) -> Result<(Self::SharedSecretKey, Self::CipherText), &'static str>
Generates a shared secret and ciphertext from an encapsulation key specific to this security parameter set.
This function utilizes the OS default random number generator and is intended to operate in constant time outside of
This function utilizes the OS default random number generator and is intended to operate in constant time outside of
rho which crosses the trust boundary in the clear. Read moreSourcefn encaps_from_seed(
&self,
seed: &[u8; 32],
) -> (Self::SharedSecretKey, Self::CipherText)
fn encaps_from_seed( &self, seed: &[u8; 32], ) -> (Self::SharedSecretKey, Self::CipherText)
Generates a shared secret and ciphertext from an encapsulation key specific to this security parameter set.
This function utilizes a provided seed (rather than a random number generator) and is intended to operate in constant time. Read more
This function utilizes a provided seed (rather than a random number generator) and is intended to operate in constant time. Read more